/*
Szymon Rusinkiewicz
Princeton University

lmsmooth.cc
Taubin lambda/mu mesh smoothing
*/

#include <stdio.h>
#include "TriMesh.h"
#include "TriMesh_algo.h"


// One iteration of umbrella-operator smoothing
void umbrella(TriMesh *mesh, float stepsize)
{
	int nv = mesh->vertices.size();
	vector<vec> disp(nv);
	int i;
	for (i = 0; i < nv; i++) {
		int nn = mesh->neighbors[i].size();
		if (!nn)
			continue;
		for (int j = 0; j < nn; j++)
			disp[i] += mesh->vertices[mesh->neighbors[i][j]];
		disp[i] /= nn;
		disp[i] -= mesh->vertices[i];
	}
	for (i = 0; i < nv; i++)
		mesh->vertices[i] += stepsize * disp[i];
}


// Several iterations of Taubin lambda/mu
void lmsmooth(TriMesh *mesh, int niters)
{
	mesh->need_neighbors();
	TriMesh::dprintf("Smoothing mesh... ");
	for (int i = 0; i < niters; i++) {
		umbrella(mesh, 0.330f);
		umbrella(mesh, -0.331f);
	}
	TriMesh::dprintf("Done.\n");
}

